############################################
#                                     Calendar -  AeroG theme                                       #
#                     By Adrien Vermeulen - Released under GPL                         #
############################################

 #this import statement allows access to the karamba functions
import karamba
import re
import os
from time import strftime

barColor = "white"
red = 255
green = 255
blue = 255
color_pid = 0
bar_pid = 0
custom_pid = 0

class CalendarModule:
	
	def __init__(self, widget, theme, color, x, y):
		self.x = x
		self.y = y
		self.c_month = int(strftime("%m"))
		self.c_year = int(strftime("%Y"))
		self.month = self.c_month
		self.year = self.c_year
		
		self.text = karamba.createText(widget, x , y, 150, 14, "")
		karamba.changeTextSize(widget, self.text, 12)
		karamba.setTextAlign(widget, self.text, "CENTER")
		karamba.changeTextColor(widget, self.text, color[0], color[1], color[2])
		
		self.bar = karamba.createImage(widget, x, y, "cal/" + theme + "/bar.png")
		self.day = karamba.createImage(widget, x, y, "cal/" + theme + "/day.png")
		self.column1 = karamba.createText(widget, x + 3, y + 14, 20, 12, "")
		self.column2 = karamba.createText(widget, x + 25, y + 14, 20, 12, "")
		self.column3 = karamba.createText(widget, x + 47, y + 14, 20, 12, "")
		self.column4 = karamba.createText(widget, x + 69, y + 14, 20, 12, "")
		self.column5 = karamba.createText(widget, x + 91, y + 14, 20, 12, "")
		self.column6 = karamba.createText(widget, x + 113, y + 14, 20, 12, "")
		self.column7 = karamba.createText(widget, x + 135, y + 14, 20, 12, "")
		
		for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
			karamba.changeTextColor(widget, meter, color[0], color[1], color[2])
	
	def displayCal(self, widget, month, year):
		self.month = month
		self.year = year
		fdate = str(month) + " " + str(year)
		
		f = os.popen('cal ' + fdate).readline()
		match = re.search("([\S]+ ?[\S]* [0-9]+)", f)
		karamba.changeText(widget, self.text, match.group(1)[0].upper() + match.group(1)[1:])
		
		c1 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b -2").read()
		c2 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 4-5").read()
		c3 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 7-8").read()
		c4 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 10-11").read()
		c5 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 13-14").read()
		c6 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 16-17").read()
		c7 = os.popen("cal -m " + fdate + " | grep -v " + str(year) + " | cut -b 19-20").read()
		
		karamba.changeText(widget, self.column1, c1)
		karamba.changeText(widget, self.column2, c2)
		karamba.changeText(widget, self.column3, c3)
		karamba.changeText(widget, self.column4, c4)
		karamba.changeText(widget, self.column5, c5)
		karamba.changeText(widget, self.column6, c6)
		karamba.changeText(widget, self.column7, c7)
		
		for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
			karamba.changeTextFont(widget, meter, "monospace")
		
		if ( month == self.c_month and year == self.c_year ):
			found = False
			column = 1
			x = self.x
			y = self.y
			date = os.popen("date +%e").read()
			date = date.replace("\n", "")
			for c in [c1, c2, c3, c4, c5, c6, c7]:
				if ( found == False ):
					match = re.search("[a-z]+\n([0-9 ]+)\n([0-9 ]+)\n([0-9 ]+)\n([0-9 ]+)\n([0-9 ]+)?(\n([0-9 ]+))?", c)
					if ( date == match.group(1) ):
						found = True
						y = 13
					elif ( date == match.group(2) ):
						found = True
						y =26
					elif ( date == match.group(3) ):
						found = True
						y = 39
					elif ( date == match.group(4) ):
						found = True
						y = 52
					elif ( date == match.group(5) ):
						found = True
						y = 65
					elif ( date == match.group(7) ):
						found = True
						y = 78
					else :
						column = column + 1
			if ( column == 1):
				x = 0
			elif ( column == 2):
				x = 22
			elif ( column == 3):
				x = 44
			elif ( column == 4):
				x = 66
			elif ( column == 5):
				x = 88
			elif ( column == 6):
				x = 110
			elif ( column == 7):
				x = 132
			karamba.moveImage(widget, self.day, self.x + x, self.y + y + 14)
			karamba.moveImage(widget, self.bar, self.x + x, self.y + 15)
			karamba.showImage(widget, self.day)
			karamba.showImage(widget, self.bar)
		else :
			karamba.hideImage(widget, self.day)
			karamba.hideImage(widget, self.bar)
	
	def update(self, widget):
		self.displayCal(widget, self.month, self.year)
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.text, color[0], color[1], color[2])
		for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
			karamba.changeTextColor(widget, meter, color[0], color[1], color[2])
	
	def updateTheme(self, widget, theme):
		karamba.setImagePath(widget, self.day, "cal/" + theme + "/day.png")
		karamba.setImagePath(widget, self.bar, "cal/" + theme + "/bar.png")
	
	def shadow(self, widget, bool):
		if bool:
			karamba.changeTextShadow(widget, self.text, 1)
			for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
				karamba.changeTextShadow(widget, meter, 1)
		else:
			karamba.changeTextShadow(widget, self.text, 0)
			for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
				karamba.changeTextShadow(widget, meter, 0)
	
	def close(self, widget):
		karamba.deleteText(widget, self.text)
		for meter in [self.column1, self.column2, self.column3, self.column4, self.column5, self.column6, self.column7]:
			karamba.deleteText(widget, meter)
		karamba.deleteImage(widget, self.bar)
		karamba.deleteImage(widget, self.day)

def updateText(widget, r, g, b):
	calendar.updateColor(widget, [r,g,b])

#this is called when your widget is initialized
def initWidget(widget):
	global red, green, blue, barColor, calendar
	
	test_barColor = karamba.readConfigEntry(widget, "bar")
	test_red = karamba.readConfigEntry(widget, "red")
	test_green = karamba.readConfigEntry(widget, "green")
	test_blue = karamba.readConfigEntry(widget, "blue")
	
	if ( test_barColor != None ):
		barColor = test_barColor
	if ( test_red != None and test_green!= None and test_blue != None ):
		red = test_red
		green = test_green
		blue = test_blue
	
	calendar = CalendarModule(widget, barColor, [red,green,blue], 19, 18)
	widgetUpdated(widget)
	#calendar.displayCal(widget, 5, 2005) #with this line you can display the month of your choice.
	karamba.redrawWidget(widget)
	
	karamba.addMenuConfigOption(widget, "barclick", "Change bar color")
	karamba.addMenuConfigOption(widget, "colorclick", "Change text color")
	karamba.setMenuConfigOption(widget, "barclick", 0)
	karamba.setMenuConfigOption(widget, "colorclick", 0)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
	calendar.update(widget)

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
	global bar_pid, color_pid

	if ( key == "barclick" ):
		karamba.setMenuConfigOption(widget, "barclick", 0)
		bar_command = ["kdialog", "--title", "Aero-G", "--menu", "Please select a bar color:", "white", "White", "green", "Green", "blue", "Blue", "yellow", "Yellow", "orange", "Orange", "red", "Red", "pink", "Pink", "lavande", "Lavande", "grey", "Grey"]
		bar_pid = karamba.executeInteractive(widget, bar_command)
	
	if ( key == "colorclick" ):
		karamba.setMenuConfigOption(widget, "colorclick", 0)
		color_command = ["kdialog", "--title", "Aero-G", "--menu", "Please select a text color:", "custom", "Custom", "white", "White", "green", "Green", "blue", "Blue", "yellow", "Yellow", "orange", "Orange", "red", "Red", "pink", "Pink", "lavande", "Lavande", "grey", "Grey"]
		color_pid = karamba.executeInteractive(widget, color_command)

def commandOutput(widget, pid, output):
	global color_pid, bar_pid, custom_pid, red, green, blue, barColor
	
	if ( pid == bar_pid ):
		barColor = output.replace('\n','')
		calendar.updateTheme(widget, barColor)
		karamba.redrawWidget(widget)
		karamba.writeConfigEntry(widget, "bar", barColor)
	
	if ( pid == color_pid ):
		textColor = output.replace('\n','')
		if ( textColor == "custom" ):
			custom_command = ["kdialog", "--title", "Aero-G", "--inputbox", "Enter a color in this format: r=xxx g=xxx b=xxx where xxx are integers in [0,255]", "r= g= b="]
			custom_pid = karamba.executeInteractive(widget, custom_command)
		else:
			if ( textColor == "white" ):
				red = 255
				green = 255
				blue = 255
			if ( textColor == "green" ):
				red = 132
				green = 255
				blue = 103
			if ( textColor == "blue" ):
				red = 122
				green = 178
				blue = 255
			if ( textColor == "yellow" ):
				red = 255
				green = 216
				blue = 0
			if ( textColor == "orange" ):
				red = 255
				green = 131
				blue = 49
			if ( textColor == "red" ):
				red = 255
				green = 73
				blue = 73
			if ( textColor == "pink" ):
				red = 255
				green = 122
				blue = 206
			if ( textColor == "lavande" ):
				red = 171
				green = 141
				blue = 255
			if ( textColor == "grey" ):
				red = 184
				green = 184
				blue = 184
			calendar.updateColor(widget, [red, green, blue])
			karamba.writeConfigEntry(widget, "red", str(red))
			karamba.writeConfigEntry(widget, "green", str(green))
			karamba.writeConfigEntry(widget, "blue", str(blue))
			karamba.redrawWidget(widget)
	
	if ( pid == custom_pid ):
		entry = output.replace('\n','')
		match = re.search("r=([0-9]+) g=([0-9]+) b=([0-9]+)", entry)
		if ( match == None ):
			error_command = ['kdialog', '--title',  'Formatting Error', '--error', 'Please enter information in the correct format.']
			karamba.executeInteractive(widget, error_command)
		else:
			red = int(match.group(1))
			green = int(match.group(2))
			blue = int(match.group(3))
			if ( red < 0 or green < 0 or blue < 0 or red > 255 or green > 255 or blue > 255 ):
				error_command = ['kdialog', '--title',  'Formatting Error', '--error', 'Please enter information in the correct format: r, g, b values may not be greater than 255 or inferior to 0.']
				karamba.executeInteractive(widget, error_command)
			else:
				calendar.updateColor(widget, [red, green, blue])
				karamba.redrawWidget(widget)
				karamba.writeConfigEntry(widget, "red", str(red))
				karamba.writeConfigEntry(widget, "green", str(green))
				karamba.writeConfigEntry(widget, "blue", str(blue))



def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass


#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    pass

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, dropText):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass

# This will be printed when the widget loads.
print "Loaded Aero-G python extension!"
