############################################
#                                          Clock  -  AeroG theme                                            #
#                     By Adrien Vermeulen - Released under GPL                        #
#                                        Thanks  to crystal clock                                           #
############################################

import karamba
import time
import math
import re

barColor = "white"
red = 184
green = 184
blue = 184
color_pid = None
bar_pid = None
custom_pid = None
showSec = False
showLogo = True

class Hand:
	def __init__(self,widget,handimage,scale,pivot_x,pivot_y,center_x,center_y):
		self.imghand=karamba.createImage(widget,0,0,handimage)
		hw=karamba.getImageWidth(widget,self.imghand)
		hh=karamba.getImageHeight(widget,self.imghand)
		r1=math.sqrt((pivot_x*pivot_x)+(pivot_y*pivot_y))
		r4=r1
		a1=math.pi-math.asin(pivot_y/r1)
		a4=2.0*math.pi-a1
		r2=math.sqrt((pivot_y*pivot_y)+((hw-pivot_x)*(hw-pivot_x)))
		r3=r2
		a2=math.asin(pivot_y/r2)
		a3=-a2
		self.a1=a1
		self.a2=a2
		self.a3=a3
		self.a4=a4
		self.r1=r1*scale
		self.r2=r2*scale
		self.r3=r3*scale
		self.r4=r4*scale
		self.center_x=center_x
		self.center_y=center_y
		self.prev=0.0	#prevents rotating etc. if not required
		
	def draw(self,widget,angle):
		if (self.prev==angle):
			return
		self.prev=angle
		karamba.rotateImage(widget,self.imghand,angle)
		a=angle/180.0*math.pi
		x1=self.center_x+self.r1*math.cos(a+self.a1)
		y1=self.center_y+self.r1*math.sin(a+self.a1)
		x2=self.center_x+self.r2*math.cos(a+self.a2)
		y2=self.center_y+self.r2*math.sin(a+self.a2)
		x3=self.center_x+self.r3*math.cos(a+self.a3)
		y3=self.center_y+self.r3*math.sin(a+self.a3)
		x4=self.center_x+self.r4*math.cos(a+self.a4)
		y4=self.center_y+self.r4*math.sin(a+self.a4)
		xl=x1
		if (x2<xl):
			xl=x2
		if (x3<xl):
			xl=x3
		if (x4<xl):
			xl=x4
		yl=y1
		if (y2<yl):
			yl=y2
		if (y3<yl):
			yl=y3
		if (y4<yl):
			yl=y4
		xm=x1
		if (x2>xm):
			xm=x2
		if (x3>xm):
			xm=x3
		if (x4>xm):
			xm=x4
		ym=y1
		if (y2>ym):
			ym=y2
		if (y3>ym):
			ym=y3
		if (y4>ym):
			ym=y4
		karamba.resizeImage(widget,self.imghand,int(xm-xl),int(ym-yl))
		karamba.moveImage(widget,self.imghand,int(xl),int(yl))
		
		
def updateText(widget, r, g, b):
	karamba.changeTextColor(widget, Text1, r, g, b)
	karamba.changeTextColor(widget, Text2, r, g, b)
	karamba.changeTextColor(widget, Text3, r, g, b)

def updateHands(widget):
	karamba.setImagePath(widget, houhand.imghand, "clock/" + barColor + "/hhand.png")
	karamba.setImagePath(widget, minhand.imghand, "clock/" + barColor + "/mhand.png")
	karamba.setImagePath(widget, sechand.imghand, "clock/" + barColor + "/shand.png")
	sechand.prev=0.0
	houhand.prev=0.0
	minhand.prev=0.0
	widgetUpdated(widget)
	karamba.redrawWidget(widget)
	

def initWidget(widget):
	global sechand,minhand,houhand, logo, Text1, Text2, Text3, showLogo, showSec
	
	logoInt = 1
	secInt = 0
	
	karamba.createBackgroundImage(widget,0,0,"clock/bg-clock.png")
	
	barColor = karamba.readConfigEntry(widget, "bar")
	red = karamba.readConfigEntry(widget, "red")
	green = karamba.readConfigEntry(widget, "green")
	blue = karamba.readConfigEntry(widget, "blue")
	testSec = karamba.readConfigEntry(widget, "sec")
	testLogo = karamba.readConfigEntry(widget, "logo")
	
	if ( testLogo != None ):
		if ( testLogo == 0 ):
			showLogo = False
			logoInt = 0
		elif ( testLogo == 1 ):
			showLogo = True
	
			
	if ( testSec != None ):
		if ( testSec == 0 ):
			showSec = False
		elif ( testSec == 1 ):
			showSec = True
			secInt = 1
			
	if ( barColor == None ):
		barColor = "white"
	
	logo = karamba.createImage(widget,38,45,"clock/logo.png")
	if ( showLogo == False ) :
		karamba.hideImage(widget, logo)
	#hand parameter: widget,hand image,shadow image,scale factor,x coord of pivot in hand image, same in y, x coord of pivot in background image, sam in y, offset for shadow
	# the scale factor shrinks the hand image (for anti-alias) so hands should be drawn larger than required
	# shadow image must be of the same size a hand image; make hand image a little larger as needed for blurring
	houhand=Hand(widget,"clock/" + barColor + "/hhand.png",0.18,0.0,5.0,75,75)
	minhand=Hand(widget,"clock/" + barColor + "/mhand.png",0.27,0.0,5.0,75,75)
	sechand=Hand(widget, "clock/" + barColor + "/shand.png",0.27,15.0,5.0,75,75) # disable me if you don't want a sec hand 	
	if ( showSec == False ) :
		karamba.hideImage(widget, sechand.imghand)
	Text1 = karamba.getThemeText(widget, "text1")
	Text2 = karamba.getThemeText(widget, "text2")
	Text3 = karamba.getThemeText(widget, "text3")
	if ( red != None and green!= None and blue != None ):
		updateText(widget, red, green, blue)
	
	karamba.createImage(widget,18,18,"clock/reflect.png") # the center should be coverd since the hands are 'dancing' a little
	
	karamba.addMenuConfigOption(widget, "barclick", "Change hands color")
	karamba.addMenuConfigOption(widget, "colorclick", "Change text color")
	karamba.addMenuConfigOption(widget, "logoclick", "Show Aero-G logo")
	karamba.addMenuConfigOption(widget, "secclick", "Show seconds hand")
	karamba.setMenuConfigOption(widget, "barclick", 0)
	karamba.setMenuConfigOption(widget, "colorclick", 0)
	karamba.setMenuConfigOption(widget, "logoclick", logoInt)
	karamba.setMenuConfigOption(widget, "secclick", secInt)
	
def widgetUpdated(widget):
	time_tuple = time.localtime()
	hours = time_tuple[3]
	minutes = time_tuple[4]
	seconds = time_tuple[5]
	houhand.draw(widget,(hours-3)*30+int(minutes/5)*2.5)
	minhand.draw(widget,(minutes-15)*6+int(seconds/15)*1.5)
	if showSec :
		sechand.draw (widget,(seconds-15)*6) # disable me if you don't want a sec hand 


def widgetClosed(widget):
    pass
    
def widgetClicked(widget, x, y, button):
    pass

def widgetMouseMoved(widget, x, y, button):
    pass

def menuItemClicked(widget, menu, id):
    pass

def menuOptionChanged(widget, key, value):
	global bar_pid, color_pid, rate_pid, showLogo, showSec
	
	if ( key == "barclick" ):
		karamba.setMenuConfigOption(widget, "barclick", 0)
		bar_command = ["kdialog", "--title", "Aero-G", "--menu", "Please select a bar color:", "white", "White", "green", "Green", "blue", "Blue", "yellow", "Yellow", "orange", "Orange", "red", "Red", "pink", "Pink", "lavande", "Lavande", "grey", "Grey"]
		bar_pid = karamba.executeInteractive(widget, bar_command)
	
	if ( key == "colorclick" ):
		karamba.setMenuConfigOption(widget, "colorclick", 0)
		color_command = ["kdialog", "--title", "Aero-G", "--menu", "Please select a text color:", "custom", "Custom", "white", "White", "green", "Green", "blue", "Blue", "yellow", "Yellow", "orange", "Orange", "red", "Red", "pink", "Pink", "lavande", "Lavande", "grey", "Grey"]
		color_pid = karamba.executeInteractive(widget, color_command)
	
	if ( key == "logoclick" ):
		logoInt = karamba.readMenuConfigOption(widget, "logoclick")
		karamba.writeConfigEntry(widget, "logo", str(logoInt))
		if ( logoInt == 0 ):
			karamba.hideImage(widget, logo)
			showLogo = False
		elif ( logoInt == 1 ):
			karamba.showImage(widget, logo)
			showLogo = True
		karamba.redrawWidget(widget)
	
	if ( key == "secclick" ):
		secInt = karamba.readMenuConfigOption(widget, "secclick")
		karamba.writeConfigEntry(widget, "sec", str(secInt))
		if ( secInt == 0 ):
			karamba.hideImage(widget, sechand.imghand)
			showSec = False
		elif ( secInt == 1 ):
			karamba.showImage(widget, sechand.imghand)
			showSec = True
			sechand.prev = 0.0
			widgetUpdated(widget)
		karamba.redrawWidget(widget)


def meterClicked(widget, meter, button):
    pass

def commandOutput(widget, pid, output):
	global color_pid, bar_pid, custom_pid, red, green, blue, barColor
	
	if ( pid == bar_pid ):
		barColor = output.replace('\n','')
		updateHands(widget)
		karamba.writeConfigEntry(widget, "bar", barColor)
	
	if ( pid == color_pid ):
		textColor = output.replace('\n','')
		if ( textColor == "custom" ):
			custom_command = ["kdialog", "--title", "Aero-G", "--inputbox", "Enter a color in this format: r=xxx g=xxx b=xxx where xxx are integers in [0,255]", "r= g= b="]
			custom_pid = karamba.executeInteractive(widget, custom_command)
		else:
			if ( textColor == "white" ):
				red = 255
				green = 255
				blue = 255
			if ( textColor == "green" ):
				red = 132
				green = 255
				blue = 103
			if ( textColor == "blue" ):
				red = 122
				green = 178
				blue = 255
			if ( textColor == "yellow" ):
				red = 255
				green = 216
				blue = 0
			if ( textColor == "orange" ):
				red = 255
				green = 131
				blue = 49
			if ( textColor == "red" ):
				red = 255
				green = 73
				blue = 73
			if ( textColor == "pink" ):
				red = 255
				green = 122
				blue = 206
			if ( textColor == "lavande" ):
				red = 171
				green = 141
				blue = 255
			if ( textColor == "grey" ):
				red = 184
				green = 184
				blue = 184
			updateText(widget, red, green, blue)
			karamba.writeConfigEntry(widget, "red", str(red))
			karamba.writeConfigEntry(widget, "green", str(green))
			karamba.writeConfigEntry(widget, "blue", str(blue))
			karamba.redrawWidget(widget)
	
	if ( pid == custom_pid ):
		entry = output.replace('\n','')
		match = re.search("r=([0-9]+) g=([0-9]+) b=([0-9]+)", entry)
		if ( match == None ):
			error_command = ['kdialog', '--title',  'Formatting Error', '--error', 'Please enter information in the correct format.']
			karamba.executeInteractive(widget, error_command)
		else:
			red = int(match.group(1))
			green = int(match.group(2))
			blue = int(match.group(3))
			if ( red < 0 or green < 0 or blue < 0 or red > 255 or green > 255 or blue > 255 ):
				error_command = ['kdialog', '--title',  'Formatting Error', '--error', 'Please enter information in the correct format: r, g, b values may not be greater than 255 or inferior to 0.']
				karamba.executeInteractive(widget, error_command)
			else:
				updateText(widget, red, green, blue)
				karamba.redrawWidget(widget)
				karamba.writeConfigEntry(widget, "red", str(red))
				karamba.writeConfigEntry(widget, "green", str(green))
				karamba.writeConfigEntry(widget, "blue", str(blue))
	

def itemDropped(widget, dropText):
    pass

def startupAdded(widget, startup):
    pass
 
def startupRemoved(widget, startup):
    pass

def taskAdded(widget, task):
    pass

def taskRemoved(widget, task):
    pass

def activeTaskChanged(widget, task):
    pass

print "Loaded my Aero-G Clock python extensions!"

