############################################
#                                            Disk -  AeroG theme                                             #
#                     By Adrien Vermeulen - Released under GPL                        #
############################################

 #this import statement allows access to the karamba functions
import karamba
import re

barColor = ""
red = 255
green = 255
blue = 255
color_pid = None
bar_pid = None
custom_pid = None

def updateText(widget, r, g, b):
	karamba.changeTextColor(widget, titleText, r, g, b)
	karamba.changeTextColor(widget, subText, r, g, b)
	karamba.changeTextColor(widget, percentText, r, g, b)

#this is called when your widget is initialized
def initWidget(widget):
	global titleText, percentText, subText, bar, ghost_bar, red, green, blue, barColor
	
	titleText = karamba.getThemeText(widget, "title")
	percentText = karamba.getThemeText(widget, "percent")
	subText = karamba.getThemeText(widget, "sub")
	bar = karamba.getThemeBar(widget, "bar")
	ghost_bar = karamba.createBar(widget, 0, 0, 0, 0, "")
	karamba.setBarSensor(widget, ghost_bar, 'sensor=memory format="%fsb"')
	
	barColor = karamba.readConfigEntry(widget, "bar")
	red = karamba.readConfigEntry(widget, "red")
	green = karamba.readConfigEntry(widget, "green")
	blue = karamba.readConfigEntry(widget, "blue")
	
	if ( barColor != None ):
		karamba.setBarImage(widget, bar, "img/gauge-" + barColor + ".png")
	if ( red != None and green!= None and blue != None ):
		updateText(widget, red, green, blue)
	
	karamba.addMenuConfigOption(widget, "barclick", "Change bar color")
	karamba.addMenuConfigOption(widget, "colorclick", "Change text color")
	karamba.setMenuConfigOption(widget, "barclick", 0)
	karamba.setMenuConfigOption(widget, "colorclick", 0)
	
	widgetUpdated(widget)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):
    used = int(karamba.getBarValue(widget, bar))
    free = int(karamba.getBarValue(widget, ghost_bar))
    karamba.changeText(widget, subText, str(free)+ "MB")
    if ( free != 0 ):
        percent = 100 * used / ( used + free )
        karamba.changeText(widget, percentText, str(percent) + "%")
    karamba.redrawWidget(widget)
    

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
	global bar_pid, color_pid

	if ( key == "barclick" ):
		karamba.setMenuConfigOption(widget, "barclick", 0)
		bar_command = ["kdialog", "--title", "Aero-G", "--menu", "Please select a bar color:", "white", "White", "dark", "Dark", "green", "Green", "blue", "Blue", "yellow", "Yellow", "orange", "Orange", "red", "Red", "pink", "Pink", "lavande", "Lavande", "grey", "Grey"]
		bar_pid = karamba.executeInteractive(widget, bar_command)
	
	if ( key == "colorclick" ):
		karamba.setMenuConfigOption(widget, "colorclick", 0)
		color_command = ["kdialog", "--title", "Aero-G", "--menu", "Please select a text color:", "custom", "Custom", "white", "White", "green", "Green", "blue", "Blue", "yellow", "Yellow", "orange", "Orange", "red", "Red", "pink", "Pink", "lavande", "Lavande", "grey", "Grey"]
		color_pid = karamba.executeInteractive(widget, color_command)

def commandOutput(widget, pid, output):
	global color_pid, bar_pid, custom_pid, red, green, blue, barColor
	
	if ( pid == bar_pid ):
		barColor = output.replace('\n','')
		karamba.setBarImage(widget, bar, "img/gauge-" + barColor + ".png")
		karamba.redrawWidget(widget)
		karamba.writeConfigEntry(widget, "bar", barColor)
	
	if ( pid == color_pid ):
		textColor = output.replace('\n','')
		if ( textColor == "custom" ):
			custom_command = ["kdialog", "--title", "Aero-G", "--inputbox", "Enter a color in this format: r=xxx g=xxx b=xxx where xxx are integers in [0,255]", "r= g= b="]
			custom_pid = karamba.executeInteractive(widget, custom_command)
		else:
			if ( textColor == "white" ):
				red = 255
				green = 255
				blue = 255
			if ( textColor == "green" ):
				red = 132
				green = 255
				blue = 103
			if ( textColor == "blue" ):
				red = 122
				green = 178
				blue = 255
			if ( textColor == "yellow" ):
				red = 255
				green = 216
				blue = 0
			if ( textColor == "orange" ):
				red = 255
				green = 131
				blue = 49
			if ( textColor == "red" ):
				red = 255
				green = 73
				blue = 73
			if ( textColor == "pink" ):
				red = 255
				green = 122
				blue = 206
			if ( textColor == "lavande" ):
				red = 171
				green = 141
				blue = 255
			if ( textColor == "grey" ):
				red = 184
				green = 184
				blue = 184
			updateText(widget, red, green, blue)
			karamba.writeConfigEntry(widget, "red", str(red))
			karamba.writeConfigEntry(widget, "green", str(green))
			karamba.writeConfigEntry(widget, "blue", str(blue))
			karamba.redrawWidget(widget)
	
	if ( pid == custom_pid ):
		entry = output.replace('\n','')
		match = re.search("r=([0-9]+) g=([0-9]+) b=([0-9]+)", entry)
		if ( match == None ):
			error_command = ['kdialog', '--title',  'Formatting Error', '--error', 'Please enter information in the correct format.']
			karamba.executeInteractive(widget, error_command)
		else:
			red = int(match.group(1))
			green = int(match.group(2))
			blue = int(match.group(3))
			if ( red < 0 or green < 0 or blue < 0 or red > 255 or green > 255 or blue > 255 ):
				error_command = ['kdialog', '--title',  'Formatting Error', '--error', 'Please enter information in the correct format: r, g, b values may not be greater than 255 or inferior to 0.']
				karamba.executeInteractive(widget, error_command)
			else:
				updateText(widget, red, green, blue)
				karamba.redrawWidget(widget)
				karamba.writeConfigEntry(widget, "red", str(red))
				karamba.writeConfigEntry(widget, "green", str(green))
				karamba.writeConfigEntry(widget, "blue", str(blue))



def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass


#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):
    pass


#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
    pass

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
def itemDropped(widget, dropText):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass

# This will be printed when the widget loads.
print "Loaded Aero-G python extension!"
 
